/*
	This file is part of lgui.

	lgui is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lgui is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lgui.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2008 Lucas Hermann Negri
*/

#include "accel.h"

/* GtkAccelGroup */

int lgui_accel_group_new(lua_State* L)
{
	GtkAccelGroup* ptr = gtk_accel_group_new();
	object_new(L, ptr, TRUE);

	return 1;
}

int lgui_accel_group_connect(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);
	luaL_checktype(L, 3, LUA_TFUNCTION);

	int ud_ref = -1;

	if(lua_gettop(L) >= 4)
	{
		luaL_checkany(L, 4);
		ud_ref = luaL_ref(L, LUA_REGISTRYINDEX);
	}

	lua_pushvalue(L, 3);
	int function_ref = luaL_ref(L, LUA_REGISTRYINDEX);

	Object* obj = lua_touserdata(L, 1);
	Data* data = g_malloc(sizeof(Data));
	data->L = L;
	data->function_ref = function_ref;
	data->ud_ref = ud_ref;

	/* We need only the user data, not the GClosure */
	data->signal_info.param_types = NULL;
	data->signal_info.n_params = 0;

	/* Build the closure */
	lua_pushstring(L, "__lobjClosureNew");
	lua_rawget(L, LUA_GLOBALSINDEX);
	lua_pushlightuserdata(L, data);
	lua_call(L, 1, 1);
	GClosure* closure = lua_touserdata(L, -1);  

	/* Temp storage, to parse the keys */
	guint key;
	GdkModifierType modifier;
	gtk_accelerator_parse(lua_tostring(L, 2), &key, &modifier);
	gtk_accel_group_connect(GTK_ACCEL_GROUP(obj->pointer), key, modifier, GTK_ACCEL_VISIBLE, closure);

 	return 0;
}

int lgui_accel_group_disconnect(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);

	Object* obj = lua_touserdata(L, 1);

	/* Temp storage, to parse the keys */
	guint key;
	GdkModifierType modifier;

	gtk_accelerator_parse(lua_tostring(L, 2), &key, &modifier);
	gboolean res = gtk_accel_group_disconnect_key(GTK_ACCEL_GROUP(obj->pointer), key, modifier);

	lua_pushboolean(L, res);
	return 1;
}
