/*
	This file is part of lgui.

	lgui is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lgui is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lgui.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2008 Lucas Hermann Negri
*/

#include "clipboard.h"

int lgui_clipboard_get(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TNUMBER);
	GdkAtom obj =  (GdkAtom)lua_tointeger(L, 1);
	
	GtkClipboard* ptr = gtk_clipboard_get(obj);
	object_new(L, ptr, FALSE);
	
	return 1;
}

int lgui_clipboard_set_text(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);
	
	Object* obj = lua_touserdata(L, 1);
	gtk_clipboard_set_text(GTK_CLIPBOARD(obj->pointer), lua_tostring(L, 2),
		lua_strlen(L, 2));
	
	return 0;
}

int lgui_clipboard_get_text(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	gboolean res = gtk_clipboard_wait_is_text_available(GTK_CLIPBOARD(obj->pointer));
	
	if(res)
	{
		gchar* text = gtk_clipboard_wait_for_text(GTK_CLIPBOARD(obj->pointer));
		lua_pushstring(L, text);
		g_free(text);
		
		return 1;
	}
	else
		return 0;
}

int lgui_clipboard_set_image(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	
	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 2);
	
	gtk_clipboard_set_image(GTK_CLIPBOARD(obj1->pointer), GDK_PIXBUF(obj2->pointer));
	
	return 0;
}

int lgui_clipboard_get_image(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	gboolean res = gtk_clipboard_wait_is_image_available(GTK_CLIPBOARD(obj->pointer));
	
	if(res)
	{
		GdkPixbuf* ptr = gtk_clipboard_wait_for_image(GTK_CLIPBOARD(obj->pointer));
		object_new(L, ptr, TRUE);
		
		return 1;
	}
	else
		return 0;
}

int lgui_clipboard_store(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	gtk_clipboard_store(GTK_CLIPBOARD(obj->pointer));
	
	return 0;
}

int lgui_clipboard_is_text_available(lua_State *L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	gboolean res = gtk_clipboard_wait_is_text_available(GTK_CLIPBOARD(obj->pointer));
	lua_pushboolean(L, res);
	
	return 1;
}

int lgui_clipboard_clear(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	gtk_clipboard_clear(GTK_CLIPBOARD(obj->pointer));
	
	return 0;
}
