/*
	This file is part of lgui.

	lgui is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lgui is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lgui.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2008 Lucas Hermann Negri
*/

#include "dnd.h"

/* GdkDragContext */

int lgui_drag_context_status(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);
	luaL_checktype(L, 3, LUA_TNUMBER);

	Object* obj = lua_touserdata(L, 1);
	gdk_drag_status(GDK_DRAG_CONTEXT(obj->pointer), (GdkDragAction)lua_tointeger(L, 2), lua_tointeger(L, 3));

	return 0;
}

int lgui_drag_context_get_source_widget(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);

	GtkWidget* ptr = gtk_drag_get_source_widget(GDK_DRAG_CONTEXT(obj->pointer));
	object_new(L, ptr, FALSE);

	return 1;
}

int lgui_drag_context_finish(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TBOOLEAN);
	luaL_checktype(L, 3, LUA_TBOOLEAN);
	luaL_checktype(L, 4, LUA_TNUMBER);

	Object* obj = lua_touserdata(L, 1);
	gtk_drag_finish(GDK_DRAG_CONTEXT(obj->pointer), lua_toboolean(L, 2), lua_toboolean(L, 3), lua_tointeger(L, 4));

	return 0;
}

/* GtkDragDest */

int lgui_drag_dest_set(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);
	
	Object* obj = lua_touserdata(L, 1);
	
	GtkTargetEntry* entry = NULL;
	GdkDragAction action = 0;
	int entry_size = 0, top = lua_gettop(L);
	
	if(!lua_isnone(L, 3) && !lua_isnil(L, 3))
	{
		luaL_checktype(L, 3, LUA_TUSERDATA);
		struct TargetEntry* obj2 = lua_touserdata(L, 3);
		entry = obj2->entry;
		entry_size = obj2->size;
	}
	
	if(top > 3)
	{
		luaL_checktype(L, 4, LUA_TNUMBER);
		action = (GdkDragAction)lua_tointeger(L, 4);
	}

	gtk_drag_dest_set(GTK_WIDGET(obj->pointer), (GdkModifierType)lua_tointeger(L, 2),
		entry, entry_size, action);

	return 0;
}

/* GtkDragSource */

int lgui_drag_source_set(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);
	
	Object* obj = lua_touserdata(L, 1);
	
	GtkTargetEntry* entry = NULL;
	GdkDragAction action = 0;
	int entry_size = 0, top = lua_gettop(L);
	
	if(!lua_isnone(L, 3) && !lua_isnil(L, 3))
	{
		luaL_checktype(L, 3, LUA_TUSERDATA);
		struct TargetEntry* obj2 = lua_touserdata(L, 3);
		entry = obj2->entry;
		entry_size = obj2->size;
	}
	
	if(top > 3)
	{
		luaL_checktype(L, 4, LUA_TNUMBER);
		action = (GdkDragAction)lua_tointeger(L, 4);
	}

	gtk_drag_source_set(GTK_WIDGET(obj->pointer), (GdkModifierType)lua_tointeger(L, 2),
		entry, entry_size, action);

	return 0;
}

int lgui_drag_source_set_icon_pixbuf(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);

	Object* obj = lua_touserdata(L, 1);
	Object* pixbuf = lua_touserdata(L, 2);

	gtk_drag_source_set_icon_pixbuf(GTK_WIDGET(obj->pointer), GDK_PIXBUF(pixbuf->pointer));

	return 0;
}

int lgui_drag_source_set_icon_stock(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	
	Object* obj = lua_touserdata(L, 1);
	gtk_drag_source_set_icon_stock(GTK_WIDGET(obj->pointer), lua_tostring(L, 2));

	return 0;
}

/* GtkTargetEntry */

static int priv_target_entry_gc(lua_State* L)
{
	#ifdef IDEBUG
	g_debug("Garbage collecting a GtkTargetEntry!");
	#endif
	
	struct TargetEntry* obj = lua_touserdata(L, 1);
	int i;
	
	for(i = 0; i < obj->size; ++i)
		g_free(obj->entry[i].target);
		
	g_free(obj->entry);
	
	return 0;	
}

int lgui_target_entry_new(lua_State* L)
{
	#ifdef IDEBUG
	g_debug("Creating a new GtkTargetEntry!");
	#endif
	
	int top = lua_gettop(L), i;
	GtkTargetEntry* entry = g_malloc(sizeof(GtkTargetEntry) * top);
	
	for(i = 1; i <= top; ++i)
	{
		luaL_checktype(L, i, LUA_TTABLE);
		
		/* Get the string */
		lua_pushnumber(L, 1);
		lua_rawget(L, i);
		entry[i - 1].target = g_strdup(lua_tostring(L, -1));
		
		lua_pushnumber(L, 2);
		lua_rawget(L, i);
		entry[i - 1].flags = lua_tointeger(L, -1);
		
		lua_pushnumber(L, 3);
		lua_rawget(L, i);
		entry[i - 1].info = lua_tointeger(L, -1);
	}
	
	struct TargetEntry* obj = lua_newuserdata(L, sizeof(struct TargetEntry));
	obj->entry = entry;
	obj->size  = top; /* The old one */
	top = lua_gettop(L); /* Get the new top */
	
	/* Shared metatable */
	if(luaL_newmetatable(L, "__lguiTaRgEtEnTrYmT"))
	{
		lua_getfield(L, LUA_GLOBALSINDEX, "__lobjRegisterSpecial");
		lua_pushvalue(L, -2);
		lua_pushstring(L, "lgui");
		lua_pushstring(L, "TargetEntry");
		lua_pushcfunction(L, priv_target_entry_gc);
		lua_call(L, 4, 0);
	}

	/* Set the object metatable */
	lua_setmetatable(L, top);
	
	return 1;
}

/* GtkSelectionData */

int lgui_selection_data_get_text(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	gchar* str = (gchar*)gtk_selection_data_get_text((GtkSelectionData*)obj->pointer);
	
	if(str)
	{
		lua_pushstring(L, str);
		g_free(str);
	}
	else
	{
		lua_pushnil(L);
	}
	
	return 1;
}

int lgui_selection_data_get_pixbuf(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	GdkPixbuf* ptr = gtk_selection_data_get_pixbuf((GtkSelectionData*)obj->pointer);
	object_new(L, ptr, TRUE);
	
	return 1;
}

int lgui_selection_data_set_text(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);
	Object* obj = lua_touserdata(L, 1);
	int top = lua_gettop(L), len = -1;
	
	if(top > 2)
	{
		luaL_checktype(L, 3, LUA_TNUMBER);
		len = lua_tointeger(L, 3);
	}
	
	gboolean reg = gtk_selection_data_set_text((GtkSelectionData*)obj->pointer,
		lua_tostring(L, 2), len);
	
	lua_pushboolean(L, reg);
	
	return 1;
}

int lgui_selection_data_set_pixbuf(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 2);
	
	gboolean reg = gtk_selection_data_set_pixbuf((GtkSelectionData*)obj1->pointer,
		GDK_PIXBUF(obj2->pointer));
	
	lua_pushboolean(L, reg);
	
	return 1;
}

int lgui_selection_data_set(lua_State* L)
{ 
    luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);
	luaL_checktype(L, 3, LUA_TSTRING);
	luaL_checktype(L, 4, LUA_TNUMBER);
	
	Object* obj = lua_touserdata(L, 1);
	GtkSelectionData* data = obj->pointer;
	
	gtk_selection_data_set(data, data->target, lua_tointeger(L, 2),
		(guchar*)lua_tostring(L, 3), lua_tointeger(L, 4));
	
	return 0;
};
