/*
	This file is part of lgui.

	lgui is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lgui is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lgui.  If not, see <http://www.gnu.org/licenses/>.

    Copyright (C) 2008 Lucas Hermann Negri
*/

#include "interface.h"

static void register_class(lua_State* L, const char* name, const char* base, const luaL_Reg* reg)
{
	/* Get the lua function */
	lua_pushstring(L, "__lguiBuildInheritance");
	lua_rawget(L, LUA_GLOBALSINDEX);

	/* The three function params */
	lua_pushstring(L, name);
	lua_pushstring(L, base);
	lua_newtable(L);
	luaL_register(L, NULL, reg);

	/* Build the inheritance */
	lua_call(L, 3, 0);
}

#ifdef HAVE_GLADE

static const struct luaL_reg glade [] =
{
	{"new", lgui_glade_xml_new},
	{"newFromBuffer", lgui_glade_xml_new_from_buffer},
	{"getWidget", lgui_glade_xml_get_widget},
	{NULL, NULL}
};

#endif

static const struct luaL_reg atom [] =
{
	{"intern", lgui_atom_intern},
	{"name", lgui_atom_name},
	{NULL, NULL}
};

static const struct luaL_reg printcontext [] =
{
	{"createPangoLayout", lgui_print_context_create_pango_layout},
	{"getDpi", lgui_print_context_get_dpi},
	{"getSize", lgui_print_context_get_size},
	{NULL, NULL}
};

static const struct luaL_reg pangofontdescription [] =
{
	{"newFromString", lgui_pango_font_description_from_string},
	{"setSize", lgui_pango_font_description_set_size},
	{NULL, NULL}
};

static const struct luaL_reg pangolayout [] =
{
	{"setText", lgui_pango_layout_set_text},
	{"setMarkup", lgui_pango_layout_set_markup},
	{"setFontDescription", lgui_pango_layout_set_font_description},
	{"setWidth", lgui_pango_layout_set_width},
	{"setHeight", lgui_pango_layout_set_height},
	{"setAligment", lgui_pango_layout_set_aligment},
	{"setIndent", lgui_pango_layout_set_indent},
	{"setJustify", lgui_pango_layout_set_justify},
	{"setSpacing", lgui_pango_layout_set_spacing},
	{"setWrap", lgui_pango_layout_set_wrap},
	{NULL, NULL}
};

static const struct luaL_reg printoperation [] =
{
	{"new", lgui_print_operation_new},
	{"run", lgui_print_operation_run},
	{NULL, NULL}
};

static const struct luaL_reg lgui [] =
{
	{"main", lgui_main},
	{"quit", lgui_quit},
	{"version", lgui_version},
	{"eventsPending", lgui_events_pending},
	{"mainIteration", lgui_main_iteration},
	{"showUri", lgui_show_uri},
	{"beep", lgui_gdk_beep},
	{NULL, NULL}
};

static const struct luaL_reg color [] =
{
	{"parse", lgui_color_parse},
	{"tostring", lgui_color_tostring},
	{NULL, NULL}
};

static const struct luaL_reg accel [] =
{
	{"new", lgui_accel_group_new},
	{"connect", lgui_accel_group_connect},
	{"disconnect", lgui_accel_group_disconnect},
	{NULL, NULL}
};

static const struct luaL_reg treeselection [] =
{
	{"setMode", lgui_tree_selection_set_mode},
	{"getMode", lgui_tree_selection_get_mode},
	{"countSelectedRows", lgui_tree_selection_count_selected_rows},
	{"selectIter", lgui_tree_selection_select_iter},
	{"unselectIter", lgui_tree_selection_unselect_iter},
	{"iterIsSelected", lgui_tree_selection_iter_is_selected},
	{"selectRange", lgui_tree_selection_select_range},
	{"unselectRange", lgui_tree_selection_unselect_range},
	{"getSelected", lgui_tree_selection_get_selected},
	{"getSelectedRows", lgui_tree_selection_get_selected_rows},
	{NULL, NULL}
};

static const struct luaL_reg treeiter [] =
{
	{"new", lgui_tree_iter_new},
	{NULL, NULL}
};

static const struct luaL_reg treemodel [] =
{
	{"getIter", lgui_tree_model_get_iter},
	{"getIterFirst", lgui_tree_model_get_iter_first},
	{"getIterFromString", lgui_tree_model_get_iter_from_string},
	{"iterNext", lgui_tree_model_iter_next},
	{"get", lgui_tree_model_get},
	{"getNColumns", lgui_tree_model_get_n_columns},
	{"getStringFromIter", lgui_tree_model_get_string_from_iter},
	{NULL, NULL}
};

static const struct luaL_reg liststore [] =
{
	{"new", lgui_list_store_new},
	{"append", lgui_list_store_append},
	{"prepend", lgui_list_store_prepend},
	{"insert", lgui_list_store_insert},
	{"remove", lgui_list_store_remove},
	{"swap", lgui_list_store_swap},
	{"set", lgui_list_store_set},
	{"seto", lgui_list_store_seto},
	{"clear", lgui_list_store_clear},
	{NULL, NULL}
};

static const struct luaL_reg treestore [] =
{
	{"new", lgui_tree_store_new},
	{"append", lgui_tree_store_append},
	{"prepend", lgui_tree_store_prepend},
	{"insert", lgui_tree_store_insert},
	{"remove", lgui_tree_store_remove},
	{"swap", lgui_tree_store_swap},
	{"set", lgui_tree_store_set},
	{"clear", lgui_tree_store_clear},
	{NULL, NULL}
};

static const struct luaL_reg cellrendereraccel [] =
{
	{"new", lgui_cell_renderer_accel_new},
	{NULL, NULL}
};

static const struct luaL_reg cellrenderercombo [] =
{
	{"new", lgui_cell_renderer_combo_new},
	{NULL, NULL}
};

static const struct luaL_reg cellrendererpixbuf [] =
{
	{"new", lgui_cell_renderer_pixbuf_new},
	{NULL, NULL}
};

static const struct luaL_reg cellrendererprogress [] =
{
	{"new", lgui_cell_renderer_progress_new},
	{NULL, NULL}
};

static const struct luaL_reg cellrendererspin [] =
{
	{"new", lgui_cell_renderer_spin_new},
	{NULL, NULL}
};

static const struct luaL_reg cellrenderertext [] =
{
	{"new", lgui_cell_renderer_text_new},
	{NULL, NULL}
};

static const struct luaL_reg cellrenderertoggle [] =
{
	{"new", lgui_cell_renderer_toggle_new},
	{NULL, NULL}
};

static const struct luaL_reg treeviewcolumn [] =
{
	{"newWithAttributes", lgui_tree_view_column_new_with_attributes},
	{"new", lgui_tree_view_column_new},
	{"setSortColumnId", lgui_tree_view_column_set_sort_column_id},
	{NULL, NULL}
};

static const struct luaL_reg treeview [] =
{
	{"new", lgui_tree_view_new},
	{"newWithModel", lgui_tree_view_new_with_model},
	{"appendColumn", lgui_tree_view_append_column},
	{"insertColumn", lgui_tree_view_insert_column},
	{"getColumn", lgui_tree_view_get_column},
	{"getSelection", lgui_tree_view_get_selection},
	{"getPathAtPos", lgui_tree_view_get_path_at_pos},
	{"convertBinWindowToTreeCoords", lgui_tree_view_convert_bin_window_to_tree_coords},
	{NULL, NULL}
};

static const struct luaL_reg pixbuf [] =
{
	{"newFromFile", lgui_pixbuf_new_from_file},
	{"newFromFileAtScale", lgui_pixbuf_new_from_file_at_scale},
	{"save", lgui_pixbuf_save},
	{"scaleSimple", lgui_pixbuf_scale_simple},
	{"rotateSimple", lgui_pixbuf_rotate_simple},
	{NULL, NULL}
};

static const struct luaL_reg editable [] =
{
	{"insertText", lgui_editable_insert_text},
	{"deleteText", lgui_editable_delete_text},
	{"getChars", lgui_editable_get_chars},
	{"selectRegion", lgui_editable_select_region},
	{"getSelectionBounds", lgui_editable_get_selection_bounds},
	{"cutClipboard", lgui_editable_cut_clipboard},
	{"copyClipboard", lgui_editable_copy_clipboard},
	{"pasteClipboard", lgui_editable_paste_clipboard},
	{"deleteSelection", lgui_editable_delete_selection},
	{"setPosition", lgui_editable_set_position},
	{"getPosition", lgui_editable_get_position},
	{NULL, NULL}
};

static const struct luaL_reg entry [] =
{
	{"new", lgui_entry_new},
	{"setCompletion", lgui_entry_set_completion},
	{NULL, NULL}
};

static const struct luaL_reg entrycompletion [] =
{
	{"new", lgui_entry_completion_new},
	{"setModel", lgui_entry_completion_set_model},
	{"setTextColumn", lgui_entry_completion_set_text_column},
	{NULL, NULL}
};

static const struct luaL_reg textview [] =
{
	{"new", lgui_text_view_new},
	{"newWithBuffer", lgui_text_view_new_with_buffer},
	{"scrollToIter", lgui_text_view_scroll_to_iter},
	{"scrollToMark", lgui_text_view_scroll_to_mark},
	{NULL, NULL}
};

static const struct luaL_reg texttagtable [] =
{
	{"add", lgui_text_tag_table_add},
	{"remove", lgui_text_tag_table_remove},
	{"lookup", lgui_text_tag_table_lookup},
	{"getSize", lgui_text_tag_table_get_size},
	{NULL, NULL}
};

static const struct luaL_reg textbuffer [] =
{
	{"new", lgui_text_buffer_new},
	{"getLineCount", lgui_text_buffer_get_line_count},
	{"getCharCount", lgui_text_buffer_get_char_count},
	{"insert", lgui_text_buffer_insert},
	{"insertWithTags", lgui_text_buffer_insert_with_tags},
	{"insertPixbuf", lgui_text_buffer_insert_pixbuf},
	{"getStartIter", lgui_text_buffer_get_start_iter},
	{"getEndIter", lgui_text_buffer_get_end_iter},
	{"getIterAtOffset", lgui_text_buffer_get_iter_at_offset},
	{"getIterAtLine", lgui_text_buffer_get_iter_at_line},
	{"getIterAtLineOffset", lgui_text_buffer_get_iter_at_line_offset},
	{"createTag", lgui_text_buffer_create_tag},
	{"selectRange", lgui_text_buffer_select_range},
	{"getIterAtMark", lgui_text_buffer_get_iter_at_mark},
	{"createMark", lgui_text_buffer_create_mark},
	{"moveMark", lgui_text_buffer_move_mark},
	{"deleteMark", lgui_text_buffer_delete_mark},
	{"delete", lgui_text_buffer_delete},
	{"getMark", lgui_text_buffer_get_mark},
	{"placeCursor", lgui_text_buffer_place_cursor},
	{"deleteSelection", lgui_text_buffer_delete_selection},
	{"getSelectionBounds", lgui_text_buffer_get_selection_bounds},
	{"cutClipboard", lgui_text_buffer_cut_clipboard},
	{"copyClipboard", lgui_text_buffer_copy_clipboard},
	{"pasteClipboard", lgui_text_buffer_paste_clipboard},
	{NULL, NULL}
};

static const struct luaL_reg textiter [] =
{
	{"new", lgui_text_iter_new},
	{"setOffset", lgui_text_iter_set_offset},
	{"getOffset", lgui_text_iter_get_offset},
	{"setLine", lgui_text_iter_set_line},
	{"getLine", lgui_text_iter_get_line},
	{"forwardSearch", lgui_text_iter_forward_search},
	{"backwardSearch", lgui_text_iter_backward_search},
	{NULL, NULL}
};

static const struct luaL_reg texttag [] =
{
	{NULL, NULL}
};

static const struct luaL_reg widget [] =
{
	{"activate", lgui_widget_activate},
	{"show", lgui_widget_show},
	{"showAll", lgui_widget_show_all},
	{"hide", lgui_widget_hide},
	{"hideAll", lgui_widget_hide_all},
	{"focus" , lgui_widget_focus},
	{"getWindow" , lgui_widget_get_window},
	{"getPos" , lgui_widget_get_pos},
	{"getSize" , lgui_widget_get_size},
	{"queueDraw" , lgui_widget_queue_draw},
	{"queueClear" , lgui_widget_queue_clear},
	{"addEvents", lgui_widget_add_events},
	{"setEvents", lgui_widget_set_events},
	{"getPointer", lgui_widget_get_pointer},
	{"renderIcon", lgui_widget_render_icon},
	{"modifyBase", lgui_widget_modify_base},
	{"modifyBg", lgui_widget_modify_bg},
	{"modifyFg", lgui_widget_modify_fg},
	{"modifyFont", lgui_widget_modify_font},
	{"modifyText", lgui_widget_modify_text},
	{"getHandle", lgui_widget_get_handle},
	{"childNotify", lgui_widget_child_notify},
	{"freezeChildNotify", lgui_widget_freeze_child_notify},
	{"thawChildNotify", lgui_widget_thaw_child_notify},
	{"realize", lgui_widget_realize},
	{"getSnapshot", lgui_widget_get_snapshot},
	{NULL, NULL}
};

static const struct luaL_reg combobox [] =
{
	{"new", lgui_combo_box_new},
	{"newWithModel", lgui_combo_box_new_with_model},
	{"getActiveIter", lgui_combo_box_get_active_iter},
	{"setModel", lgui_combo_box_set_model},
	{"newText", lgui_combo_box_new_text},
	{"appendText", lgui_combo_box_append_text},
	{"prependText", lgui_combo_box_prepend_text},
	{"getActiveText", lgui_combo_box_get_active_text},
	{"removeText", lgui_combo_box_remove_text},
	{"insertText", lgui_combo_box_insert_text},
	{NULL, NULL}
};

static const struct luaL_reg comboboxentry [] =
{
	{"new", lgui_combo_box_entry_new},
	{"newText", lgui_combo_box_entry_new_text},
	{"newWithModel" , lgui_combo_box_entry_new_with_model},
	{NULL, NULL}
};

static const struct luaL_reg statusbar [] =
{
	{"new", lgui_statusbar_new},
	{"getContextId", lgui_statusbar_get_context_id},
	{"push" , lgui_statusbar_push},
	{"pop" , lgui_statusbar_pop},
	{"remove" , lgui_statusbar_remove},
	{NULL, NULL}
};

static const struct luaL_reg toolbar [] =
{
	{"new", lgui_toolbar_new},
	{"insert", lgui_toolbar_insert},
	{NULL, NULL}
};

static const struct luaL_reg toolbutton [] =
{
	{"new", lgui_tool_button_new},
	{"newFromStock", lgui_tool_button_new_from_stock},
	{NULL, NULL}
};

static const struct luaL_reg toolitem [] =
{
	{"new", lgui_tool_item_new},
	{NULL, NULL}
};

static const struct luaL_reg menutoolbutton [] =
{
	{"new", lgui_menu_tool_button_new},
	{"newFromStock", lgui_menu_tool_button_new_from_stock},
	{NULL, NULL}
};

static const struct luaL_reg separatortoolitem [] =
{
	{"new", lgui_separator_tool_item_new},
	{NULL, NULL}
};

static const struct luaL_reg label [] =
{
	{"new", lgui_label_new},
	{"newWithMnemonic", lgui_label_new_with_mnemonic},
	{"setLineWrap", lgui_label_set_line_wrap},
	{"setTextWithMnemonic", lgui_label_set_text_with_mnemonic},
	{NULL, NULL}
};

static const struct luaL_reg scrolledwindow [] =
{
	{"new", lgui_scrolled_window_new},
	{NULL, NULL}
};

static const struct luaL_reg adjustment [] =
{
	{"new", lgui_adjustment_new},
	{NULL, NULL}
};

static const struct luaL_reg statusicon [] =
{
	{"new", lgui_status_icon_new},
	{"setTooltip", lgui_status_icon_set_tooltip},
	{"isEmbedded", lgui_status_icon_is_embedded},
	{NULL, NULL}
};

static const struct luaL_reg progressbar [] =
{
	{"new", lgui_progress_bar_new},
	{"pulse", lgui_progress_bar_pulse},
	{NULL, NULL}
};

static const struct luaL_reg frame [] =
{
	{"new", lgui_frame_new},
	{NULL, NULL}
};

static const struct luaL_reg eventbox [] =
{
	{"new", lgui_event_box_new},
	{NULL, NULL}
};

static const struct luaL_reg drawingarea [] =
{
	{"new", lgui_drawing_area_new},
	{NULL, NULL}
};

static const struct luaL_reg image [] =
{
	{"new", lgui_image_new},
	{NULL, NULL}
};

static const struct luaL_reg container [] =
{
	{"add", lgui_container_add},
	{"remove", lgui_container_remove},
	{"childSet", lgui_container_child_set},
	{"childGet", lgui_container_child_get},
	{"getChildren", lgui_container_get_children},
	{"reallocateRedraws", lgui_container_set_reallocate_redraws},
	{NULL, NULL}
};

static const struct luaL_reg box [] =
{
	{"packStart", lgui_box_pack_start},
	{"packEnd", lgui_box_pack_end},
	{NULL, NULL}
};

static const struct luaL_reg vbox [] =
{
	{"new", lgui_vbox_new},
	{NULL, NULL}
};

static const struct luaL_reg hbox [] =
{
	{"new", lgui_hbox_new},
	{NULL, NULL}
};

static const struct luaL_reg table [] =
{
	{"new", lgui_table_new},
	{"resize", lgui_table_resize},
	{"attach", lgui_table_attach},
	{"attachDefaults", lgui_table_attach_defaults},
	{NULL, NULL}
};

static const struct luaL_reg window [] =
{
	{"new", lgui_window_new},
	{"addAccelGroup", lgui_window_add_accel_group},
	{"removeAccelGroup", lgui_window_remove_accel_group},
	{"maximize", lgui_window_maximize},
	{"unmaximize", lgui_window_unmaximize},
	{"fullscreen", lgui_window_fullscreen},
	{"unfullscreen", lgui_window_unfullscreen},
	{"iconify", lgui_window_iconify},
	{"deiconify", lgui_window_deiconify},
	{"destroy", lgui_window_destroy},
	{"resize", lgui_window_resize},
	{"move", lgui_window_move},
	{NULL, NULL}
};

static const struct luaL_reg togglebutton [] =
{
	{"new", lgui_toggle_button_new},
	{"newWithMnemonic", lgui_toggle_button_new_with_mnemonic},
	{NULL, NULL}
};

static const struct luaL_reg checkbutton [] =
{
	{"new", lgui_check_button_new},
	{"newWithMnemonic", lgui_check_button_new_with_mnemonic},
	{NULL, NULL}
};

static const struct luaL_reg radiobutton [] =
{
	{"new", lgui_radio_button_new},
	{"newWithMnemonic", lgui_radio_button_new_with_mnemonic},
	{"newFromWidget", lgui_radio_button_new_from_widget},
	{"newWithMnemonicFromWidget", lgui_radio_button_new_with_mnemonic_from_widget},
	{NULL, NULL}
};

static const struct luaL_reg button [] =
{
	{"new", lgui_button_new},
	{"newWithMnemonic", lgui_button_new_with_mnemonic},
	{"newFromStock", lgui_button_new_from_stock},
	{NULL, NULL}
};

static const struct luaL_reg menubar [] =
{
	{"new", lgui_menu_bar_new},
	{NULL, NULL}
};

static const struct luaL_reg menu [] =
{
	{"new", lgui_menu_new},
	{"popup", lgui_menu_popup},
	{"popupFromStatusIcon", lgui_menu_popup_from_status_icon},
	{NULL, NULL}
};

static const struct luaL_reg menushell [] =
{
	{"append", lgui_menu_shell_append},
	{"insert", lgui_menu_shell_insert},
	{NULL, NULL}
};

static const struct luaL_reg menuitem [] =
{
	{"new", lgui_menu_item_new},
	{"newWithMnemonic", lgui_menu_item_new_with_mnemonic},
	{"setSubmenu", lgui_menu_item_set_submenu},
	{NULL, NULL}
};

static const struct luaL_reg imagemenuitem [] =
{
	{"new", lgui_image_menu_item_new},
	{"newFromStock", lgui_image_menu_item_new_from_stock},
	{NULL, NULL}
};

static const struct luaL_reg separatormenuitem [] =
{
	{"new", lgui_separator_menu_item_new},
	{NULL, NULL}
};

static const struct luaL_reg checkmenuitem [] =
{
	{"newWithMnemonic", lgui_check_menu_item_new_with_mnemonic},
	{NULL, NULL}
};

static const struct luaL_reg tearoffmenuitem [] =
{
	{"new", lgui_tearoff_menu_item_new},
	{NULL, NULL}
};

static const struct luaL_reg dialog [] =
{
	{"new", lgui_dialog_new},
	{"run", lgui_dialog_run},
	{"addButton", lgui_dialog_add_button},
	{"getVBox", lgui_dialog_get_vbox},
	{"response", lgui_dialog_response},
	{NULL, NULL}
};

static const struct luaL_reg filefilter [] =
{
	{"new", lgui_file_filter_new},
	{"setName", lgui_file_filter_set_name},
	{"getName", lgui_file_filter_get_name},
	{"addMimetype", lgui_file_filter_add_mimetype},
	{"addPattern", lgui_file_filter_add_pattern},
	{"addPixbufFormats", lgui_file_filter_add_pixbuf_formats},
	{NULL, NULL}
};

static const struct luaL_reg filechooserdialog [] =
{
	{"new", lgui_file_chooser_dialog_new},
	{NULL, NULL}
};

static const struct luaL_reg filechooser [] =
{
	{"addFilter", lgui_file_chooser_add_filter},
	{"removeFilter", lgui_file_chooser_remove_filter},
	{"setFilename", lgui_file_chooser_set_filename},
	{"getFilename", lgui_file_chooser_get_filename},
	{"getFilenames", lgui_file_chooser_get_filenames},
	{"selectFilename", lgui_file_chooser_select_filename},
	{"unselectFilename", lgui_file_chooser_unselect_filename},
	{"selectAll", lgui_file_chooser_select_all},
	{"unselectAll", lgui_file_chooser_unselect_all},
	{"setCurrentFolder", lgui_file_chooser_set_current_folder},
	{"getCurrentFolder", lgui_file_chooser_get_current_folder},
	{NULL, NULL}
};

static const struct luaL_reg colorselectiondialog [] =
{
	{"new", lgui_color_selection_dialog_new},
	{"getChooser", lgui_color_selection_dialog_get_chooser},
	{NULL, NULL}
};

static const struct luaL_reg colorselection [] =
{
	{NULL, NULL}
};

static const struct luaL_reg fontselectiondialog [] =
{
	{"new", lgui_font_selection_dialog_new},
	{"getChooser", lgui_font_selection_dialog_get_chooser},
	{NULL, NULL}
};

static const struct luaL_reg fontselection [] =
{
	{NULL, NULL}
};

static const struct luaL_reg aboutdialog [] =
{
	{"new", lgui_about_dialog_new},
	{"setEmailHook", lgui_about_dialog_set_email_hook},
	{"setUrlHook", lgui_about_dialog_set_url_hook},
	{NULL, NULL}
};

static const struct luaL_reg treemodelfilter [] =
{
	{"new", lgui_tree_model_filter_new},
	{"setVisibleFunction", lgui_tree_model_filter_set_visible_function},
	{"setVisibleColumn", lgui_tree_model_filter_set_visible_column},
	{"refilter", lgui_tree_model_filter_refilter},
	{NULL, NULL}
};

static const struct luaL_reg treesortable [] =
{
	{"setSortColumnId", lgui_tree_sortable_set_sort_column_id},
	{"getSortColumnId", lgui_tree_sortable_get_sort_column_id},\
	{NULL, NULL}
};

static const struct luaL_reg vscale [] =
{
	{"new", lgui_vscale_new},
	{"newWithRange", lgui_vscale_new_with_range},
	{NULL, NULL}
};

static const struct luaL_reg hscale [] =
{
	{"new", lgui_hscale_new},
	{"newWithRange", lgui_hscale_new_with_range},
	{NULL, NULL}
};

static const struct luaL_reg dragcontext [] =
{
	{"status", lgui_drag_context_status},
	{"getSourceWidget", lgui_drag_context_get_source_widget},
	{"finish", lgui_drag_context_finish},
	{NULL, NULL}
};

static const struct luaL_reg dragdest [] =
{
	{"set", lgui_drag_dest_set},
	{NULL, NULL}
};

static const struct luaL_reg dragsource [] =
{
	{"set", lgui_drag_source_set},
	{"setIconPixbuf" , lgui_drag_source_set_icon_pixbuf},
	{"setIconStock" , lgui_drag_source_set_icon_stock},
	{NULL, NULL}
};

static const struct luaL_reg event [] =
{
	{"buttonGet", lgui_event_button_get},
	{"keyGet", lgui_event_key_get},
	{NULL, NULL}
};

static const struct luaL_reg iconview [] =
{
	{"new", lgui_icon_view_new},
	{"newWithModel", lgui_icon_view_new_with_model},
	{"getSelectedItems", lgui_icon_view_get_selected_items},
	{"getPathAtPos", lgui_icon_view_get_path_at_pos},
	{"selectAll", lgui_icon_view_select_all},
	{"unselectAll", lgui_icon_view_unselect_all},
	{"convertWidgetToBinWindowCoords", lgui_icon_view_convert_widget_to_bin_window_coords},
	{NULL, NULL}
};

static const struct luaL_reg calendar [] =
{
	{"new", lgui_calendar_new},
	{"markDay", lgui_calendar_mark_day},
	{"unmarkDay", lgui_calendar_unmark_day},
	{"clearMarks", lgui_calendar_clear_marks},
	{NULL, NULL}
};

static const struct luaL_reg notebook [] =
{
	{"new", lgui_notebook_new},
	{"insertPage", lgui_notebook_insert_page},
	{"insertPageMenu", lgui_notebook_insert_page_menu},
	{"removePage", lgui_notebook_remove_page},
	{NULL, NULL}
};

static const struct luaL_reg assistant [] =
{
	{"new", lgui_assistant_new},
	{"insertPage", lgui_assistant_insert_page},
	{NULL, NULL}
};

static const struct luaL_reg vpaned [] =
{
	{"new", lgui_vpaned_new},
	{NULL, NULL}
};

static const struct luaL_reg hpaned [] =
{
	{"new", lgui_hpaned_new},
	{NULL, NULL}
};

static const struct luaL_reg expander [] =
{
	{"newWithMnemonic", lgui_expander_new_with_mnemonic},
	{NULL, NULL}
};

static const struct luaL_reg colorbutton [] =
{
	{"new", lgui_color_button_new},
	{NULL, NULL}
};

static const struct luaL_reg fontbutton [] =
{
	{"new", lgui_font_button_new},
	{NULL, NULL}
};

static const struct luaL_reg filechooserbutton [] =
{
	{"new", lgui_file_chooser_button_new},
	{NULL, NULL}
};

static const struct luaL_reg scalebutton [] =
{
	{"new", lgui_scale_button_new},
	{NULL, NULL}
};

static const struct luaL_reg volumebutton [] =
{
	{"new", lgui_volume_button_new},
	{NULL, NULL}
};

static const struct luaL_reg spinbutton [] =
{
	{"new", lgui_spin_button_new},
	{"newWithRange", lgui_spin_button_new_with_range},
	{"update", lgui_spin_button_update},
	{NULL, NULL}
};

static const struct luaL_reg celllayout [] =
{
	{"addAttribute", lgui_cell_layout_add_attribute},
	{"clearAttributes", lgui_cell_layout_clear_attributes},
	{"clear", lgui_cell_layout_clear},
	{"packStart", lgui_cell_layout_pack_start},
	{"packEnd", lgui_cell_layout_pack_end},
	{"reorder", lgui_cell_layout_reorder},
	{NULL, NULL}
};

static const struct luaL_reg alignment [] =
{
	{"new", lgui_alignment_new},
	{NULL, NULL}
};

static const struct luaL_reg sizegroup [] =
{
	{"new", lgui_size_group_new},
	{"addWidget", lgui_size_group_add_widget},
	{"removeWidget", lgui_size_group_remove_widget},
	{NULL, NULL}
};

static const struct luaL_reg handlebox [] =
{
	{"new", lgui_handle_box_new},
	{NULL, NULL}
};

static const struct luaL_reg vseparator [] =
{
	{"new", lgui_vseparator_new},
	{NULL, NULL}
};

static const struct luaL_reg hseparator [] =
{
	{"new", lgui_hseparator_new},
	{NULL, NULL}
};

static const struct luaL_reg hbuttonbox [] =
{
	{"new", lgui_hbutton_box_new},
	{NULL, NULL}
};

static const struct luaL_reg vbuttonbox [] =
{
	{"new", lgui_vbutton_box_new},
	{NULL, NULL}
};

static const struct luaL_reg builder [] =
{
	{"new", lgui_builder_new},
	{"getObject", lgui_builder_get_object},
	{"getObjects", lgui_builder_get_objects},
	{"addFromFile", lgui_builder_add_from_file},
	{"addFromString", lgui_builder_add_from_string},
	{NULL, NULL}
};

static const struct luaL_reg arrow [] =
{
	{"new", lgui_arrow_new},
	{NULL, NULL}
};

static const struct luaL_reg screen [] =
{
	{"getDefault", lgui_screen_get_default},
	{"getNMonitors", lgui_screen_get_n_monitors},
	{"getNumber", lgui_screen_get_number},
	{"getSize", lgui_screen_get_size},
	{"isComposited", lgui_screen_is_composited},
	{NULL, NULL}
};

static const struct luaL_reg recentinfo [] =
{
	{"getURI", lgui_recent_info_get_uri},
	{"getDisplayName", lgui_recent_info_get_display_name},
	{"getDescription", lgui_recent_info_get_description},
	{"getMimeType", lgui_recent_info_get_mime_type},
	{"getAdded", lgui_recent_info_get_added},
	{"getModified", lgui_recent_info_get_modified},
	{"getVisited", lgui_recent_info_get_visited},
	{"getPrivateHint", lgui_recent_info_get_private_hint},
	{"getShortName", lgui_recent_info_get_short_name},
	{"exists", lgui_recent_info_exists},
	{"getApplications", lgui_recent_info_get_applications},
	{"lastApplication", lgui_recent_info_last_application},
	{NULL, NULL}
};

static const struct luaL_reg recentmanager [] =
{
	{"new", lgui_recent_manager_new},
	{"getDefault", lgui_recent_manager_get_default},
	{"addItem", lgui_recent_manager_add_item},
	{"removeItem", lgui_recent_manager_remove_item},
	{"lookupItem", lgui_recent_manager_lookup_item},
	{"hasItem", lgui_recent_manager_has_item},
	{"moveItem", lgui_recent_manager_move_item},
	{"purgeItems", lgui_recent_manager_purge_items},
	{"getItems", lgui_recent_manager_get_items},
	{NULL, NULL}
};

static const struct luaL_reg recentfilter [] =
{
	{"new", lgui_recent_filter_new},
	{"getName", lgui_recent_filter_get_name},
	{"setName", lgui_recent_filter_set_name},
	{"addMimeType", lgui_recent_filter_add_mime_type},
	{"addPattern", lgui_recent_filter_add_pattern},
	{"addApplication", lgui_recent_filter_add_application},
	{"addAge", lgui_recent_filter_add_age},
	{"addPixbufFormats", lgui_recent_filter_add_pixbuf_formats},
	{NULL, NULL}
};

static const struct luaL_reg recentchooser [] =
{
	{"addFilter", lgui_recent_chooser_add_filter},
	{"removeFilter", lgui_recent_chooser_remove_filter},
	{"selectURI", lgui_recent_chooser_select_uri},
	{"unselectURI", lgui_recent_chooser_unselect_uri},
	{"selectAll", lgui_recent_chooser_select_all},
	{"unselectAll", lgui_recent_chooser_unselect_all},
	{"setCurrentURI", lgui_recent_chooser_set_current_uri},
	{"getCurrentURI", lgui_recent_chooser_get_current_uri},
	{"getCurrentItem", lgui_recent_chooser_get_current_item},
	{"getItems", lgui_recent_chooser_get_items},
	{NULL, NULL}
};

static const struct luaL_reg recentchooserdialog [] =
{
	{"new", lgui_recent_chooser_dialog_new},
	{"newForManager", lgui_recent_chooser_dialog_new_for_manager},
	{NULL, NULL}
};

static const struct luaL_reg recentchoosermenu [] =
{
	{"new", lgui_recent_chooser_menu_new},
	{"newForManager", lgui_recent_chooser_menu_new_for_manager},
	{NULL, NULL}
};

static const struct luaL_reg action [] =
{
	{"new", lgui_action_new},
	{"activate", lgui_action_activate},
	{"connectAccelerator", lgui_action_connect_accelerator},
	{"disconnectAccelerator", lgui_action_disconnect_accelerator},
	{"connectProxy", lgui_action_connect_proxy},
	{"disconnectProxy", lgui_action_disconnect_proxy},
	{"createIcon", lgui_action_create_icon},
	{"createMenuItem", lgui_action_create_menu_item},
	{"createToolItem", lgui_action_create_tool_item},
	{"getProxies", lgui_action_get_proxies},
	{"setAccelGroup", lgui_action_set_accel_group},
	{NULL, NULL}
};

static const struct luaL_reg recentaction [] =
{
	{"new", lgui_recent_action_new},
	{"newForManager", lgui_recent_action_new_for_manager},
	{NULL, NULL}
};

static const struct luaL_reg toggleaction [] =
{
	{"new", lgui_toggle_action_new},
	{"toggled", lgui_toggle_action_toggled},
	{NULL, NULL}
};

static const struct luaL_reg radioaction [] =
{
	{"new", lgui_radio_action_new},
	{NULL, NULL}
};

static const struct luaL_reg actiongroup [] =
{
	{"new", lgui_action_group_new},
	{"addAction", lgui_action_group_add_action},
	{"getAction", lgui_action_group_get_action},
	{"removeAction", lgui_action_group_remove_action},
	{"addActionWithAccel", lgui_action_group_add_action_with_accel},
	{"listActions", lgui_action_group_list_actions},
	{NULL, NULL}
};

static const struct luaL_reg tooltip [] =
{
	{"setCustom", lgui_tooltip_set_custom},
	{"setIcon", lgui_tooltip_set_icon},
	{"setIconFromStock", lgui_tooltip_set_icon_from_stock},
	{"setMarkup", lgui_tooltip_set_markup},
	{"setText", lgui_tooltip_set_text},
	{NULL, NULL}
};

static const struct luaL_reg clipboard [] =
{
	{"get", lgui_clipboard_get},
	{"setText", lgui_clipboard_set_text},
	{"getText", lgui_clipboard_get_text},
	{"setImage", lgui_clipboard_set_image},
	{"getImage", lgui_clipboard_get_image},
	{"store", lgui_clipboard_store},
	{"isTextAvailable", lgui_clipboard_is_text_available},
	{"clear", lgui_clipboard_clear},
	{NULL, NULL}
};

static const struct luaL_reg targetentry [] =
{
	{"new", lgui_target_entry_new},
	{NULL, NULL}
};

static const struct luaL_reg selectiondata [] =
{
	{"getPixbuf", lgui_selection_data_get_pixbuf},
	{"getText", lgui_selection_data_get_text},
	{"setPixbuf", lgui_selection_data_set_pixbuf},
	{"setText", lgui_selection_data_set_text},
	{"set", lgui_selection_data_set},
	{NULL, NULL}
};

static const struct luaL_reg gdkwindow [] =
{
	{"clear", lgui_gdk_window_clear},
	{"clearArea", lgui_gdk_window_clear_area},
	{"getPointer", lgui_gdk_window_get_pointer},
	{"setCursor", lgui_gdk_window_set_cursor},
	{NULL, NULL}
};

static const struct luaL_reg cursor [] =
{
	{"new", lgui_gdk_cursor_new},
	{NULL, NULL}
};

static void priv_init_gtk(lua_State* L)
{
	int argc = 1;
	char** argv = g_malloc(argc * sizeof(char*));

	lua_pushstring(L, "arg");
	lua_rawget(L, LUA_GLOBALSINDEX);

	if(!lua_istable(L, -1))
		argv[0] = "lgui";
	else
	{
		lua_pushnumber(L, 0);
		lua_rawget(L, -2);
		argv[0] = (char*)luaL_optstring(L, -1, "lgui");
	}

	gtk_init(&argc, &argv);
	g_free(argv);
}

int luaopen__lgui(lua_State *L)
{
	/* Init GTK+ */
	priv_init_gtk(L);

	/* Register the base library */
	luaL_register(L, LIB_NAME, lgui);

	/* Register some types (push needed to prevent compiler optimizations)*/
	lua_pushinteger(L, gdk_pixbuf_get_type());

	/* Register the base Object */
	lua_getfield(L, LUA_GLOBALSINDEX, "__lguiRegisterBaseObject");
	lua_call(L, 0, 0);

	/* Register the classes */
	#ifdef HAVE_GLADE
	register_class(L, "Glade", "Object", glade);
	#endif

	register_class(L, "Clipboard", "Object", clipboard);
	register_class(L, "Tooltip", "Object", tooltip);
	register_class(L, "ActionGroup", "Object", actiongroup);
	register_class(L, "Action", "Object", action);
		register_class(L, "RecentAction", "Action", recentaction);
		register_class(L, "ToggleAction", "Action", toggleaction);
			register_class(L, "RadioAction", "ToggleAction", radioaction);
	register_class(L, "Pixbuf", "Object", pixbuf);
	register_class(L, "RecentFilter", "Object", recentfilter);
	register_class(L, "PangoLayout", "Object", pangolayout);
	register_class(L, "RecentManager", "Object", recentmanager);
	register_class(L, "Screen", "Object", screen);
	register_class(L, "Builder", "Object", builder);
	register_class(L, "SizeGroup", "Object", sizegroup);
	register_class(L, "TextTagTable", "Object", texttagtable);
	register_class(L, "PrintContext", "Object", printcontext);
	register_class(L, "PrintOperation", "Object", printoperation);
	register_class(L, "EntryCompletion", "Object", entrycompletion);
	register_class(L, "AccelGroup", "Object", accel);
	register_class(L, "StatusIcon", "Object", statusicon);
	register_class(L, "TextBuffer", "Object", textbuffer);
	register_class(L, "TreeSelection", "Object", treeselection);
	register_class(L, "CellRendererAccel", "Object", cellrendereraccel);
	register_class(L, "CellRendererCombo", "Object", cellrenderercombo);
	register_class(L, "CellRendererPixbuf", "Object", cellrendererpixbuf);
	register_class(L, "CellRendererProgress", "Object", cellrendererprogress);
	register_class(L, "CellRendererSpin", "Object", cellrendererspin);
	register_class(L, "CellRendererToggle", "Object", cellrenderertoggle);
	register_class(L, "CellRendererText", "Object", cellrenderertext);
	register_class(L, "TreeViewColumn", "Object", treeviewcolumn);
	register_class(L, "Adjustment", "Object", adjustment);
	register_class(L, "FileFilter", "Object", filefilter);
	register_class(L, "Widget", "Object", widget);
		register_class(L, "Arrow", "Widget", arrow);
		register_class(L, "VSeparator", "Widget", vseparator);
		register_class(L, "HSeparator", "Widget", hseparator);
		register_class(L, "Calendar", "Widget", calendar);
		register_class(L, "VScale", "Widget", vscale);
		register_class(L, "HScale", "Widget", hscale);
		register_class(L, "Entry", "Widget", entry);
			register_class(L, "SpinButton", "Entry", spinbutton);
		register_class(L, "TextView", "Widget", textview);
		register_class(L, "Label", "Widget", label);
		register_class(L, "ProgressBar", "Widget", progressbar);
		register_class(L, "DrawingArea", "Widget", drawingarea);
		register_class(L, "Image", "Widget", image);
		register_class(L, "Container", "Widget", container);
			register_class(L, "MenuItem", "Container", menuitem);
				register_class(L, "ImageMenuItem", "MenuItem", imagemenuitem);
				register_class(L, "SeparatorMenuItem", "MenuItem", separatormenuitem);
				register_class(L, "CheckMenuItem", "MenuItem", checkmenuitem);
				register_class(L, "TearoffMenuItem", "MenuItem", tearoffmenuitem);
			register_class(L, "TreeView", "Container", treeview);
			register_class(L, "ToolItem", "Container", toolitem);
				register_class(L, "ToolButton", "ToolItem", toolbutton);
					register_class(L, "MenuToolbutton", "ToolButton", menutoolbutton);
			register_class(L, "SeparatorToolItem", "ToolItem", separatortoolitem);
			register_class(L, "ScrolledWindow", "Container", scrolledwindow);
			register_class(L, "Alignment", "Container", alignment);
			register_class(L, "Frame", "Container", frame);
			register_class(L, "Window", "Container", window);
				register_class(L, "Dialog", "Window", dialog);
				register_class(L, "Assistant", "Window", assistant);
					register_class(L, "RecentChooserDialog", "Dialog", recentchooserdialog);
					register_class(L, "FileChooserDialog", "Dialog", filechooserdialog);
					register_class(L, "ColorSelectionDialog", "Dialog", colorselectiondialog);
					register_class(L, "FontSelectionDialog", "Dialog", fontselectiondialog);
					register_class(L, "AboutDialog", "Dialog", aboutdialog);
			register_class(L, "Button", "Container", button);
				register_class(L, "ScaleButton", "Button", scalebutton);
					register_class(L, "VolumeButton", "ScaleButton", volumebutton);
				register_class(L, "ColorButton", "Button", colorbutton);
				register_class(L, "FontButton", "Button", fontbutton);
				register_class(L, "FileChooserButton", "Button", filechooserbutton);
				register_class(L, "ToggleButton", "Button", togglebutton);
					register_class(L, "CheckButton", "ToggleButton", checkbutton);
						register_class(L, "RadioButton", "CheckButton", radiobutton);
			register_class(L, "ComboBox", "Container", combobox);
				register_class(L, "ComboBoxEntry", "ComboBox", comboboxentry);
			register_class(L, "HandleBox", "Container", handlebox);
			register_class(L, "Expander", "Container", expander);
			register_class(L, "EventBox", "Container", eventbox);
			register_class(L, "VPaned", "Container", vpaned);
			register_class(L, "HPaned", "Container", hpaned);
			register_class(L, "Notebook", "Container", notebook);
			register_class(L, "IconView", "Container", iconview);
			register_class(L, "Statusbar", "Container", statusbar);
			register_class(L, "Toolbar", "Container", toolbar);
			register_class(L, "Table", "Container", table);
			register_class(L, "MenuShell", "Container", menushell);
				register_class(L, "MenuBar", "MenuShell", menubar);
				register_class(L, "Menu", "MenuShell", menu);
					register_class(L, "RecentChooserMenu", "Menu", recentchoosermenu);
			register_class(L, "Box", "Container", box);
				register_class(L, "HBox", "Box", hbox);
				register_class(L, "VBox", "Box", vbox);
				register_class(L, "HButtonBox", "Box", hbuttonbox);
				register_class(L, "VButtonBox", "Box", vbuttonbox);
					register_class(L, "FontSelection", "VBox", fontselection);
					register_class(L, "ColorSelection", "VBox", colorselection);
	register_class(L, "TreeModelFilter", "Object", treemodelfilter);
	register_class(L, "ListStore", "Object", liststore);
	register_class(L, "TreeStore", "Object", treestore);
	register_class(L, "GdkWindow", "Object", gdkwindow);

	/* Interfaces */

	register_class(L, "RecentChooser", NULL, recentchooser);
	register_class(L, "Editable", NULL, editable);
	register_class(L, "FileChooser", NULL, filechooser);
	register_class(L, "CellLayout", NULL, celllayout);
	register_class(L, "TreeModel", NULL, treemodel);
		register_class(L, "TreeSortable", "TreeModel", treesortable);
	register_class(L, "DragDest", NULL, dragdest);
	register_class(L, "DragSource", NULL, dragsource);

	/* Callback arguments */

	register_class(L, "DragContext", NULL, dragcontext);
	register_class(L, "Event", NULL, event);

	/* Simple structs */

	register_class(L, "TreeIter", NULL, treeiter);
	register_class(L, "TextIter", NULL, textiter);

	/* Non GObject objects! */

	register_class(L, "Atom", NULL, atom);
	register_class(L, "Cursor", NULL, cursor);
	register_class(L, "RecentInfo", NULL, recentinfo);
	register_class(L, "PangoFontDescription", NULL, pangofontdescription);
	register_class(L, "Color", NULL, color);
	register_class(L, "TargetEntry", NULL, targetentry);
	register_class(L, "SelectionData", NULL, selectiondata);

	return 1;
}
