/*
	This file is part of lgui.

	lgui is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lgui is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lgui.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2008 Lucas Hermann Negri
*/

#ifndef LGUI_LGUI
#define LGUI_LGUI

#define LIB_NAME "lgui"
#define LIB_VERSION "0.21"

/* Lua headers */
#include <lua.h>
#include <lauxlib.h>

/* GTK and others */
#include <gtk/gtk.h>

/* lobj */
#include <lobj/types.h>

/* C */
#include <string.h>

/* Platform specific */
#ifdef G_OS_WIN32
	#define LIB_NAME_GTK "gtk-win32-2.0.dll"
	#include <gdk/gdkwin32.h> /* Yes, probabily it can't hold a 64bit pointer */ 
	#define GET_HANDLER(win) ((ptrdiff_t)GDK_WINDOW_HWND(win))
#else /* Here, assume a X11 backend */
	#define LIB_NAME_GTK "libgtk-x11-2.0."G_MODULE_SUFFIX
	#include <gdk/gdkx.h>
	#define GET_HANDLER(win) (GDK_DRAWABLE_XID(win))
#endif

/**
 * Main loop.
 */
int lgui_main(lua_State* L);

/**
 * Quit from the main loop.
 */
int lgui_quit(lua_State* L);

/**
 * Get if there are any pending events
 */
int lgui_events_pending(lua_State* L);

/**
 * Run a single event iteration. Can block if there
 * isn't a pending event.
 */
int lgui_main_iteration(lua_State* L);

/**
 * Return the library version.
 */
int lgui_version(lua_State* L);

/**
 * Creates a new Object.
 *
 * @param L 			lua_State to use
 * @param pointer 		Pointer to the real GObject
 * @param need_unrefGC 	If the object needs an extra unref
 */
gboolean object_new(lua_State* L, gpointer pointer, gboolean need_unref);

/**
 * Creates a new Struct.
 *
 * @param L 			lua_State to use
 * @param pointer 		Pointer to the real gtk struct
 * @param need_unref	If a manual free must be called at gc
 */
gboolean struct_new(lua_State* L, gpointer pointer, gboolean need_unref);

/**
 * Frees a callback.
 *
 * @param user_data 	Userdata that contains the callback info
 */
void callback_free(gpointer user_data);

#endif
