/*
	This file is part of lgui.

	lgui is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lgui is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lgui.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2008 Lucas Hermann Negri
*/

#include "static.h"

/* GtkScrolledWindow */

int lgui_scrolled_window_new(lua_State* L)
{
	GtkWidget* ptr = NULL;

	/* User passed the adjustments */
	if(lua_gettop(L) >= 2)
	{
		luaL_checktype(L, 1, LUA_TUSERDATA);
		luaL_checktype(L, 2, LUA_TUSERDATA);
		Object* ha = lua_touserdata(L, 1);
		Object* va = lua_touserdata(L, 2);

		ptr = gtk_scrolled_window_new(GTK_ADJUSTMENT(ha->pointer), GTK_ADJUSTMENT(va->pointer));
	}
	/* Default */
	else
	{
		ptr = gtk_scrolled_window_new(NULL, NULL);
	}

	object_new(L, ptr, FALSE);
	return 1;
}

/* GtkAdjustment */

int lgui_adjustment_new(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TNUMBER);
	luaL_checktype(L, 2, LUA_TNUMBER);
	luaL_checktype(L, 3, LUA_TNUMBER);
	luaL_checktype(L, 4, LUA_TNUMBER);
	luaL_checktype(L, 5, LUA_TNUMBER);
	luaL_checktype(L, 6, LUA_TNUMBER);

	GtkObject* ptr = gtk_adjustment_new(lua_tonumber(L, 1), lua_tonumber(L, 2),
	lua_tonumber(L, 3), lua_tonumber(L, 4), lua_tonumber(L, 5), lua_tonumber(L, 6));

	object_new(L, ptr, FALSE);
	return 1;
}

/* GtkProgressBar */

int lgui_progress_bar_new(lua_State* L)
{
	GtkWidget* ptr = gtk_progress_bar_new();
	object_new(L, ptr, FALSE);

	return 1;
}

int lgui_progress_bar_pulse(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);

	Object* obj = lua_touserdata(L, 1);
	gtk_progress_bar_pulse(GTK_PROGRESS_BAR(obj->pointer));

	return 0;
}

/* GtkFrame */

int lgui_frame_new(lua_State* L)
{
	int top = lua_gettop(L);
	char* label = NULL;
	
	if(top > 0)
	{
		luaL_checktype(L, 1, LUA_TSTRING);
		label = (char*)lua_tostring(L, 1);
	}
	
	GtkWidget* ptr = gtk_frame_new(label);
	object_new(L, ptr, FALSE);

	return 1;
}

/* GtkEventBox */

int lgui_event_box_new(lua_State* L)
{
	GtkWidget* ptr = gtk_event_box_new();
	object_new(L, ptr, FALSE);

	return 1;
}

/* GtkDrawingArea */

int lgui_drawing_area_new(lua_State* L)
{
	GtkWidget* ptr = gtk_drawing_area_new();
	object_new(L, ptr, FALSE);

	return 1;
}

/* GtkImage */

int lgui_image_new(lua_State* L)
{
	GtkWidget* ptr = gtk_image_new();
	object_new(L, ptr, FALSE);

	return 1;
}

/* GtkVPaned */

int lgui_vpaned_new(lua_State* L)
{
	GtkWidget* ptr = gtk_vpaned_new();
	object_new(L, ptr, FALSE);

	return 1;
}

/* GtkHPaned */

int lgui_hpaned_new(lua_State* L)
{
	GtkWidget* ptr = gtk_hpaned_new();
	object_new(L, ptr, FALSE);

	return 1;
}

/* GtkExpander */

int lgui_expander_new_with_mnemonic(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TSTRING);
	GtkWidget* ptr = gtk_expander_new_with_mnemonic(lua_tostring(L, 1));
	object_new(L, ptr, FALSE);

	return 1;
}

/* GtkAlignment */

int lgui_alignment_new(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TNUMBER);
	luaL_checktype(L, 2, LUA_TNUMBER);
	luaL_checktype(L, 3, LUA_TNUMBER);
	luaL_checktype(L, 4, LUA_TNUMBER);
	
	GtkWidget* ptr = gtk_alignment_new(lua_tonumber(L, 1), lua_tonumber(L, 2),
	 lua_tonumber(L, 3), lua_tonumber(L, 4));
	
	object_new(L, ptr, FALSE); 
	 
	return 1;
}

/* GtkHandleBox */

int lgui_handle_box_new(lua_State* L)
{
	GtkWidget* ptr = gtk_handle_box_new();
	
	object_new(L, ptr, FALSE);
	return 1;
}

/* GtkSeparator */

int lgui_vseparator_new(lua_State* L)
{
	GtkWidget* ptr = gtk_vseparator_new();
	object_new(L, ptr, FALSE);
	return 1;
}

int lgui_hseparator_new(lua_State* L)
{
	GtkWidget* ptr = gtk_hseparator_new();
	object_new(L, ptr, FALSE);
	return 1;
}

/* GtkArrow */

int lgui_arrow_new(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TNUMBER);
	luaL_checktype(L, 2, LUA_TNUMBER);
	
	GtkWidget* ptr = gtk_arrow_new((GtkArrowType)lua_tointeger(L, 1), 
		(GtkShadowType)lua_tointeger(L, 2));
	object_new(L, ptr, FALSE);	
	
	return 1;
}

/* Misc */

int lgui_show_uri(lua_State* L)
{
	#if GTK_MINOR_VERSION >= 13
	luaL_checktype(L, 1, LUA_TSTRING);
	GError* errors = NULL;
	gboolean res = gtk_show_uri(NULL, lua_tostring(L, 1), GDK_CURRENT_TIME, &errors);
	lua_pushboolean(L, res);
	
	if(errors)
	{
		lua_pushstring(L, errors->message);
		g_error_free(errors);
		return 2;
	}
	
	#else
	lua_pushboolean(L, FALSE);
	#endif
	
	return 1;
}
