/*
	This file is part of lgui.

	lgui is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lgui is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lgui.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2008 Lucas Hermann Negri
*/

#include "treeview.h"

/* GtkTreeViewColumn */

int lgui_tree_view_column_new_with_attributes(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TSTRING);
	luaL_checktype(L, 2, LUA_TUSERDATA);

	int size = lua_gettop(L), i;
	Object* obj = lua_touserdata(L, 2);

	/* Create the column */
	GtkTreeViewColumn* ptr = gtk_tree_view_column_new_with_attributes(lua_tostring(L, 1), GTK_CELL_RENDERER(obj->pointer), NULL);

	/* Add other attributes */
	for(i = 3; i <= size; i += 2)
	{
		luaL_checktype(L, i, LUA_TSTRING);
		luaL_checktype(L, i + 1, LUA_TNUMBER);

		gtk_tree_view_column_add_attribute(ptr, GTK_CELL_RENDERER(obj->pointer),
		lua_tostring(L, i),  lua_tonumber(L, i + 1));
	}

	object_new(L, ptr, FALSE);

	return 1;
}

int lgui_tree_view_column_new(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TSTRING);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	luaL_checktype(L, 3, LUA_TSTRING);
	luaL_checktype(L, 4, LUA_TNUMBER);
	int top = lua_gettop(L);

	Object* obj = lua_touserdata(L, 2);
	gboolean flag = FALSE;
	gint id = lua_tointeger(L, 4);

	if(top > 4)
	{
		luaL_checktype(L, 5, LUA_TBOOLEAN);
		flag = lua_toboolean(L, 5);
	} 

	/* Create the column */
	GtkTreeViewColumn* ptr = gtk_tree_view_column_new_with_attributes(lua_tostring(L, 1),
	GTK_CELL_RENDERER(obj->pointer), lua_tostring(L, 3), id, NULL);

	if(flag)
	{
		gtk_tree_view_column_set_reorderable(ptr, TRUE);
		gtk_tree_view_column_set_resizable(ptr, TRUE);
		gtk_tree_view_column_set_expand(ptr, TRUE);
		gtk_tree_view_column_set_sort_column_id(ptr, id);
	}

	object_new(L, ptr, FALSE);

	return 1;
}

int lgui_tree_view_column_set_sort_column_id(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);

	Object* obj = lua_touserdata(L, 1);
	gtk_tree_view_column_set_sort_column_id(GTK_TREE_VIEW_COLUMN(obj->pointer), lua_tointeger(L, 2));

	return 0;
}


/* GtkTreeView */

int lgui_tree_view_new(lua_State* L)
{
	GtkWidget* ptr = gtk_tree_view_new();
	object_new(L, ptr, FALSE);

	return 1;
}

int lgui_tree_view_new_with_model(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	GtkWidget* ptr = gtk_tree_view_new_with_model(GTK_TREE_MODEL(obj->pointer));
	object_new(L, ptr, FALSE);

	return 1;
}

int lgui_tree_view_append_column(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);

	Object* obj = lua_touserdata(L, 1);
	int size = lua_gettop(L), i, res;

	for(i = 2; i <= size; i++)
	{
		luaL_checktype(L, i, LUA_TUSERDATA);
		Object* stc = lua_touserdata(L, i);
		res = gtk_tree_view_append_column(GTK_TREE_VIEW(obj->pointer), GTK_TREE_VIEW_COLUMN(stc->pointer));
		lua_pushnumber(L, res);
	}

	return size - 1;
}

int lgui_tree_view_insert_column(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	luaL_checktype(L, 3, LUA_TNUMBER);

	Object* obj = lua_touserdata(L, 1);
	Object* stc = lua_touserdata(L, 2);

	gint res = gtk_tree_view_insert_column(GTK_TREE_VIEW(obj->pointer), GTK_TREE_VIEW_COLUMN(stc->pointer),
	lua_tointeger(L, 3));
	lua_pushnumber(L, res);

	return 1;
}

int lgui_tree_view_remove_column(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);

	Object* obj = lua_touserdata(L, 1);
	int size = lua_gettop(L), i, res;

	for(i = 2; i <= size; i++)
	{
		luaL_checktype(L, i, LUA_TUSERDATA);
		Object* stc = lua_touserdata(L, i);
		res = gtk_tree_view_remove_column(GTK_TREE_VIEW(obj->pointer), GTK_TREE_VIEW_COLUMN(stc->pointer));
		lua_pushnumber(L, res);
	}

	return size - 1;
}

int lgui_tree_view_get_column(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);

	Object* obj = lua_touserdata(L, 1);
	GtkTreeViewColumn* res = gtk_tree_view_get_column(GTK_TREE_VIEW(obj->pointer), lua_tonumber(L, 2));
	object_new(L, res, FALSE);

	return 1;
}

int lgui_tree_view_get_selection(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);

	Object* obj = lua_touserdata(L, 1);
	GtkTreeSelection* res = gtk_tree_view_get_selection(GTK_TREE_VIEW(obj->pointer));
	object_new(L, res, FALSE);

	return 1;
}

int lgui_tree_view_get_path_at_pos(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);
	luaL_checktype(L, 3, LUA_TNUMBER);
	
	Object* obj = lua_touserdata(L, 1);
	
	GtkTreePath* path = NULL;
	GtkTreeViewColumn* col = NULL;
	gint cell_x, cell_y;
	
	gboolean res = gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(obj->pointer),
		lua_tointeger(L, 2), lua_tointeger(L, 3), &path, &col, &cell_x, &cell_y);
	
	if(path == NULL)
	{
		lua_pushboolean(L, FALSE);
		return 1;
	}
	
	gchar* spath = gtk_tree_path_to_string(path);
	
	lua_pushboolean(L, res);
	lua_pushstring(L, spath);
	object_new(L, col, FALSE);
	lua_pushinteger(L, cell_x);
	lua_pushinteger(L, cell_y);
	
	gtk_tree_path_free(path);
	g_free(spath);
	
	return 5;
}

int lgui_tree_view_convert_bin_window_to_tree_coords(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);
	luaL_checktype(L, 3, LUA_TNUMBER);
	
	Object* obj = lua_touserdata(L, 1);
	
	gint nX, nY;
	
	gtk_tree_view_convert_bin_window_to_tree_coords(GTK_TREE_VIEW(obj->pointer),
		lua_tointeger(L, 2), lua_tointeger(L, 3), &nX, &nY);
		
	lua_pushinteger(L, nX);
	lua_pushinteger(L, nY);
	
	return 2;
}
